from python_framework.api.src.enumeration.HttpStatus import HttpStatus

class Key:
    NAME = 'name'
    LICENSE_NAME = NAME
    TAG_NAME = NAME
    CONTACT_NAME = NAME

    SWAGGER_VERSION = 'swagger'
    INFO = 'info'
    TITLE = 'title'
    DESCRIPTION = 'description'
    TERMS_OF_SERVICE = 'termsOfService'
    VERSION = 'version'
    CONTACT = 'contact'
    EMAIL = 'email'
    LICENSE = 'license'
    URL = 'url'
    HOST = 'host'
    BASE_PATH = 'basePath'
    TAGS = 'tags'
    EXTERNAL_DOCS = 'externalDocs'
    SCHEMA = 'schema'
    SCHEMES = 'schemes'
    DEFINITIONS = 'definitions'
    PROPERTIES = 'properties'
    EXAMPLE = 'example'
    PROPERTY = 'property'
    TYPE = 'type'
    REQUIRED = 'required'
    PATHS = 'paths'
    SUMARY = 'sumary'
    OPERATION_ID = 'operationId'
    CONSUMES = 'consumes'
    PRODUCES = 'produces'
    RESPONSES = 'responses'
    SECURITY = 'security'
    PARAMETERS = 'parameters'
    IN = 'in'
    S_REF = '$ref'
    CATEGORY = 'category'
    ITEMS = 'items'
    DEFAULT_STATUS_CODE = HttpStatus.OK
